#include <sys/stat.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "ansi.h"
#include "check_conflicts.h"
#include "check_gnome_icon.h"
#include "check_installed.h"
#include "check_request.h"
#include "common_errors.h"
#include "concatenate.h"
#include "deltree.h"
#include "file_exists.h"
#include "file_move.h"
#include "makedir.h"
#include "parse_category.h"
#include "parse_info.h"
#include "parse_options.h"
#include "runcommand.h"
#include "setup_main.h"
#include "setup_finalize.h"
#include "untar.h"



void remove_tempfiles(int sig) {
  printf("- Rimozione file temporanei su messaggio d'errore\n");
  // This is a signal handler, hoping nobody has changed the path ...
  deltree( "install" );
  deltree( "incoming" );
  if ((sig>0)&&(sig<50)) {
    printERROR("setup: ricevuto signal %i \n",sig);
    normal_color();
  }
  exit(sig);
}



// Installa, Aggiorna, Ripristina il pacchetto
int setup_install(TSetupOptions *lpopt) {
  int retval=0;		// Valore di ritorno
  char tempfile[256];	// Nome del file INFO
  char cmdline[256];
  TPackageInfo pkginfo;
  char c;
        
  //printf("setup: Entro in setup_install\n");

  // Link signals
  signal( SIGTERM, remove_tempfiles );
  signal( SIGINT,  remove_tempfiles );
  signal( SIGQUIT, remove_tempfiles );
  signal( SIGKILL, remove_tempfiles );
  signal( SIGHUP,  remove_tempfiles );

  // Untar solo la directory install del pacchetto
  if (lpopt->verbose > 0 ) printf( "- Estrazione informazioni di installazione dal pacchetto\n" );
  retval = untar( lpopt, UNTARMASK_INSTALL );
  if (retval != 0 ) remove_tempfiles(retval);

  // Untar solo la directory install del pacchetto
  if (lpopt->verbose > 0 ) printf( "- Analisi informazioni dal pacchetto\n" );
  sprintf( tempfile, "%s/install/INFO", lpopt->targetdir );
  retval = parse_info( tempfile, &pkginfo );
  if (retval != 0 ) remove_tempfiles(retval);
  if (lpopt->debug == 1 ) debug_info(&pkginfo);

  // Aggiusta categoria: usa la categoria da dove stai installando
  // se il pacchetto e nella directory 'packages' altrimenti usa
  // la categoria nel file 'INFO' del pacchetto.
  if (lpopt->verbose > 0 ) printf( "- Verifica categoria di appartenenza del pacchetto\n" );
  parse_category( lpopt->sourcedir, &pkginfo );

  // TODO: controllo architettura
  // check_arch( &pkginfo );  

  // Controllo se pacchetto e' gia' installato
  if (lpopt->verbose > 0 ) printf( "- Controllo installazione pacchetto\n" );
  retval = check_installed(lpopt);
  if (retval != 0 ) remove_tempfiles(retval);

  // TODO: controllo spazio libero
  // if (lpopt->action == ACTION_INSTALL) check_freespace( &pkginfo );  

  // Gestione Conflitti
  if (lpopt->action == ACTION_INSTALL) {
    if (lpopt->verbose > 0 ) printf( "- Controllo conflitti pacchetto\n" );
    sprintf( tempfile, "%s/install/CONFLICTS", lpopt->targetdir );
    retval = check_conflicts( tempfile, lpopt );
    if (retval != 0 ) remove_tempfiles(retval);
  }

  // Banner di avvio
  printNOTICE("- Installazione del pacchetto %s da %s\n", lpopt->fullname, pkginfo.category);
  color(FG_BLUE);
  printf("- %s\n", pkginfo.summary );
  normal_color();


  // Esegue PRE-INSTALL (non per restore)
  if (lpopt->action != ACTION_RESTORE ) {
    sprintf( tempfile, "%s/install/PRE-INSTALL", lpopt->targetdir );
    if ( 0 == file_exists(tempfile) ) {
      printSUCCESS("- Eseguo lo script PRE-INSTALL\n");
      normal_color();
      retval = runshell(tempfile);
      chdir(lpopt->targetdir);
      if (retval!=0) {
        printERROR("Errore durante l'esecuzione di PRE-INSTALL (%i).\n", retval );
	normal_color();
	printf("  Premi INVIO per continuare");
	c = getchar();
      }
    }
  }


  // Untar del pacchetto
  if (lpopt->verbose > 0 ) printf( "- Estrazione dei files dal pacchetto\n" );
  retval = untar( lpopt, UNTARMASK_ALL );
  if (retval != 0 ) {
    printERROR("Errore durante l'estrazione dei files (%i)\n", retval );
    normal_color();
    remove_tempfiles(retval);
  }

  // ldconfig when in interactive mode
  if (lpopt->interactive == 1) {
    printSUCCESS("- Configurazione delle librerie dinamiche ... ");
    normal_color();
    runshell("ldconfig");
    chdir(lpopt->targetdir);
    printf("Ok\n");
  }


  // Esegue INSTALL (non per restore)
  if (lpopt->action != ACTION_RESTORE ) {
    sprintf( tempfile, "%s/install/INSTALL", lpopt->targetdir );
    if ( 0 == file_exists(tempfile) ) {
      printSUCCESS("- Eseguo lo script INSTALL\n");
      normal_color();
      retval = runshell(tempfile);
      chdir(lpopt->targetdir);
      if (retval!=0) {
        printERROR("Errore durante l'esecuzione di INSTALL (%i).\n", retval );
	normal_color();
	printf("  Premi INVIO per continuare");
	c = getchar();
      }
    }
  }

  // Schedule POST-INSTALL script
  if ( lpopt->action != ACTION_RESTORE ) {
    sprintf( tempfile, "%s/install/POST-INSTALL", lpopt->targetdir );
    if ( 0 == file_exists(tempfile) ) {
      if (lpopt->verbose > 0 ) printf( "- Schedulazione dello script di POST-INSTALL\n" );
      //sprintf( cmdline, "mv %s %s/%s.post-install", tempfile, lpopt->spoolroot, lpopt->fullname );
      //retval = runshell(cmdline);
      //chdir(lpopt->targetdir);
      //if (retval!=0) printf("* System(%i): %s\n", retval, cmdline );
      sprintf( cmdline, "%s/%s.post-install", lpopt->spoolroot, lpopt->fullname );
      if ( 0 != file_move(tempfile, cmdline ) ) {
        printERROR("Errore durante lo spostamento dello script di POST-INSTALL\n");
	normal_color();
	remove_tempfiles(ERRORE_IO);
      }
    }
  }

  // Schedule X-POST-INSTALL script
  if ( lpopt->action != ACTION_RESTORE ) {
    sprintf( tempfile, "%s/install/X-POST-INSTALL", lpopt->targetdir );
    if ( 0 == file_exists(tempfile) ) {
      if (lpopt->verbose > 0 ) printf( "- Schedulazione dello script di X-POST-INSTALL\n" );
      //sprintf( cmdline, "mv %s %s/%s.x-post-install", tempfile, lpopt->spoolroot, lpopt->fullname );
      //retval = system(cmdline);
      //chdir(lpopt->targetdir);
      //if (retval!=0) printf("* System(%i): %s\n", retval, cmdline );
      sprintf( cmdline, "%s/%s.x-post-install", lpopt->spoolroot, lpopt->fullname );
      if ( 0 != file_move(tempfile, cmdline ) ) {
        printERROR("Errore durante lo spostamento dello script di X-POST-INSTALL\n");
	normal_color();
	remove_tempfiles(ERRORE_IO);
      }
    }
  }

  // Schedule Dependencies
  retval = check_request(lpopt);
  if (retval != 0 ) {
    printERROR("Errore durante il parsing di REQUEST (%i)\n", retval );
    normal_color();
    remove_tempfiles(retval);
  }

  // Schedule install-menu
  retval = check_gnome_icon(lpopt);
  if (retval != 0 ) {
    printERROR("Errore durante il controllo delle icone Gnome (%i)\n", retval );
    normal_color();
    remove_tempfiles(retval);
  }

  // Registrazione Pacchetto (TODO)
  if (lpopt->verbose > 0 ) printf( "- Registrazione informazioni pacchetto\n" );
  if ( 0 == file_exists(lpopt->spooldir) ) {
    deltree(lpopt->spooldir);
  }
  sprintf( cmdline, "mv install/ %s",lpopt->spooldir);
  retval = system(cmdline);
  chdir(lpopt->targetdir);
  if (retval!=0) printf("* Ritorno (%i): %s\n", retval, cmdline );

  // Clean-Up
  if (lpopt->verbose > 0 ) printf("- Rimozione dei file temporanei\n");
  chdir(lpopt->targetdir);
  deltree( "install" );
  deltree( "incoming" );

  // Well done, call finalize on interactive installation
  printSUCCESS( "Installazione pacchetto completata con successo\n" );
  normal_color();
  // Run --finalize in interactive mode
  if ( lpopt->interactive == 1 ) {
    lpopt->noheader = 1;
    lpopt->action = ACTION_FINALIZE;
    return setup_finalize( lpopt );
  }  
  
  // Uscita con error code = 0
  //remove_tempfiles(0);  
  return 0;
}
